# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.


"""Minimal Flask application example.

SPHINX-START

First install Invenio-Logging, setup the application and load
fixture data by running:

.. code-block:: console

   $ pip install -e .[all]
   $ cd examples
   $ ./app-setup.sh
   $ ./app-fixtures.sh

Next, start the development server:

.. code-block:: console

   $ export FLASK_APP=app.py FLASK_DEBUG=1
   $ flask run

and open the example application in your browser:

.. code-block:: console

    $ open http://127.0.0.1:5000/

You should now be able to see that an error message have been logged to the
console.

To reset the example application run:

.. code-block:: console

    $ ./app-teardown.sh

SPHINX-END
"""

from __future__ import absolute_import, print_function

from flask import Flask

from invenio_logging.console import InvenioLoggingConsole

# Create Flask application
app = Flask(__name__)
InvenioLoggingConsole(app)


@app.route("/")
def index():
    """Log error."""
    app.logger.error("Example error")
    return "Welcome to Invenio-Logging!"



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio base logging module."""

from __future__ import absolute_import, print_function

import logging


class InvenioLoggingBase(object):
    """Invenio-Logging extension for console."""

    def __init__(self, app=None):
        """Extension initialization.

        :param app: An instance of :class:`~flask.Flask`.
        """
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Initialize app.

        :param app: An instance of :class:`~flask.Flask`.
        """

    @staticmethod
    def capture_pywarnings(handler):
        """Log python system warnings."""
        logger = logging.getLogger("py.warnings")
        # Check for previously installed handlers.
        for h in logger.handlers:
            if isinstance(h, handler.__class__):
                return
        logger.addHandler(handler)
        logger.setLevel(logging.WARNING)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Module providing logging capabilities.

Invenio-Logging is a core component of Invenio responsible for configuring
the Flask application logger. The Flask application logger exposes the standard
Python logging interface for creating log records and installing predefined
handlers.

The following logger extensions exists:

- :class:`~invenio_logging.console.InvenioLoggingConsole`
- :class:`~invenio_logging.fs.InvenioLoggingFS`
- :class:`~invenio_logging.sentry.InvenioLoggingSentry`

Initialization
--------------
First make sure you have Flask application with Click support (meaning
Flask 0.11+):

>>> from flask import Flask
>>> app = Flask('myapp')

Next, initialize your logging extensions:

>>> from invenio_logging.console import InvenioLoggingConsole
>>> from invenio_logging.fs import InvenioLoggingFS
>>> console = InvenioLoggingConsole(app)
>>> fs = InvenioLoggingFS(app)

In order for the following examples to work, you need to work within an
Flask application context so let's push one:

>>> app.app_context().push()

Logging
-------
All application logging should happen via the Flask application logger to
ensure that you only have to configure one logger in order to route log
messages to your desired logging infrastructure.

In Invenio modules this is easily achieved by simply using the Flask current
application context:

>>> from flask import current_app
>>> current_app.logger.debug('Where am I?')
>>> current_app.logger.info('Hello world!')
>>> current_app.logger.warning('Be carefull with overlogging.')
>>> current_app.logger.error('Connection could not be initialized.')
>>> current_app.logger.exception('You should not divide by zero!')

Note that ``logger.exception()`` will automatically include the exception
stacktrace in the log record, which each log handler may decide to include or
not in its output.

You may also manually include exception information in the logger using the
``exc_info`` keyword argument:

>>> current_app.logger.critical("My message", exc_info=1)

Warnings
--------
Warnings are useful to alert developers and system administrators about
possible problems, e.g. usage of obsolete modules, deprecated APIs etc.

By default warnings are only sent to the console when the Flask application
is in debug mode. This can however be changed via the configuration variables:
:data:`invenio_logging.config.LOGGING_CONSOLE_PYWARNINGS`,
:data:`invenio_logging.config.LOGGING_FS_PYWARNINGS` and
:data:`invenio_logging.config.LOGGING_SENTRY_PYWARNINGS`

>>> import warnings
>>> warnings.warn('This feature is deprecated.', PendingDeprecationWarning)

For more information about logging please see:

 * https://flask.palletsprojects.com/en/1.1.x/quickstart/#logging
 * https://docs.python.org/3/library/logging.html

"""

from __future__ import absolute_import, print_function

__version__ = "1.3.3"

__all__ = ("__version__",)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Pytest configuration."""

from __future__ import absolute_import, print_function

import logging
import shutil
import tempfile

import pytest
from mock import Mock, patch


@pytest.yield_fixture()
def tmppath():
    """Make a temporary directory."""
    tmpdir = tempfile.mkdtemp()
    yield tmpdir
    shutil.rmtree(tmpdir)


@pytest.yield_fixture()
def pywarnlogger():
    """Rest the py.warnings logger."""
    logger = logging.getLogger("py.warnings")
    yield logger
    logger.handlers = []


@pytest.yield_fixture()
def sentry_emit():
    """Mock Sentry emit."""
    from raven.handlers.logging import SentryHandler

    with patch.object(SentryHandler, "emit") as sentry_emit:
        yield sentry_emit



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Console logging test."""

from __future__ import absolute_import, print_function

import logging

from flask import Flask

from invenio_logging.console import InvenioLoggingConsole


def test_init():
    """Test extension initialization."""
    app = Flask("testapp")
    InvenioLoggingConsole(app)
    assert "invenio-logging-console" in app.extensions

    ext = InvenioLoggingConsole()
    app = Flask("testapp")
    ext.init_app(app)
    assert "invenio-logging-console" in app.extensions

    app = Flask("testapp")
    app.config.update(dict(LOGGING_CONSOLE=False))
    InvenioLoggingConsole(app)
    assert "invenio-logging-console" not in app.extensions


def test_conf():
    """Test extension initialization."""
    app = Flask("testapp")
    InvenioLoggingConsole(app)
    assert app.config["LOGGING_CONSOLE_PYWARNINGS"] is False
    assert app.config["LOGGING_CONSOLE_LEVEL"] is None
    app = Flask("testapp")
    app.config.update(dict(DEBUG=True, LOGGING_CONSOLE_LEVEL="ERROR"))
    InvenioLoggingConsole(app)
    assert app.config["LOGGING_CONSOLE_PYWARNINGS"] is True
    app.logger.level == logging.ERROR


def test_warnings(pywarnlogger):
    """Test extension initialization."""
    app = Flask("testapp")
    InvenioLoggingConsole(app)
    assert logging.StreamHandler in [x.__class__ for x in pywarnlogger.handlers]



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio filesystem logging module.

This extension is automatically installed via ``invenio_base.apps`` and
``invenio_base.api_apps`` entry points.
"""

from __future__ import absolute_import, print_function

import logging
import sys
from logging.handlers import RotatingFileHandler
from os.path import dirname, exists

from . import config
from .ext import InvenioLoggingBase
from .utils import add_request_id_filter


class InvenioLoggingFS(InvenioLoggingBase):
    """Invenio-Logging extension. Filesystem handler."""

    def init_app(self, app):
        """Flask application initialization."""
        self.init_config(app)
        if app.config["LOGGING_FS_LOGFILE"] is None:
            return
        self.install_handler(app)
        app.extensions["invenio-logging-fs"] = self

    def init_config(self, app):
        """Initialize config."""
        app.config.setdefault("LOGGING_FS_LEVEL", "DEBUG" if app.debug else "WARNING")
        for k in dir(config):
            if k.startswith("LOGGING_FS"):
                app.config.setdefault(k, getattr(config, k))

        # Support injecting instance path and/or sys.prefix
        if app.config["LOGGING_FS_LOGFILE"] is not None:
            app.config["LOGGING_FS_LOGFILE"] = app.config["LOGGING_FS_LOGFILE"].format(
                instance_path=app.instance_path,
                sys_prefix=sys.prefix,
            )

    def install_handler(self, app):
        """Install log handler on Flask application."""
        # Check if directory exists.
        basedir = dirname(app.config["LOGGING_FS_LOGFILE"])
        if not exists(basedir):
            raise ValueError("Log directory {0} does not exist.".format(basedir))

        handler = RotatingFileHandler(
            app.config["LOGGING_FS_LOGFILE"],
            backupCount=app.config["LOGGING_FS_BACKUPCOUNT"],
            maxBytes=app.config["LOGGING_FS_MAXBYTES"],
            delay=True,
        )
        handler.setFormatter(
            logging.Formatter(
                "%(asctime)s %(levelname)s: %(message)s " "[in %(pathname)s:%(lineno)d]"
            )
        )
        handler.setLevel(app.config["LOGGING_FS_LEVEL"])

        # Add handler to application logger
        app.logger.addHandler(handler)

        if app.config["LOGGING_FS_PYWARNINGS"]:
            self.capture_pywarnings(handler)

        # Add request_id to log record
        app.logger.addFilter(add_request_id_filter)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Sentry logging tests."""

from __future__ import absolute_import, print_function

import logging
import warnings

import httpretty
import pytest
from celery.utils.log import get_task_logger
from flask import Flask, g, request
from flask_celeryext import create_celery_app
from flask_login import LoginManager, UserMixin, login_user
from mock import Mock, patch


def test_init():
    """Test initialization."""
    from invenio_logging.sentry import InvenioLoggingSentry

    app = Flask("testapp")
    InvenioLoggingSentry(app)
    assert "invenio-logging-sentry" not in app.extensions

    app = Flask("testapp")
    app.config["SENTRY_DSN"] = "http://user:pw@localhost/0"
    InvenioLoggingSentry(app)
    assert "invenio-logging-sentry" in app.extensions
    if app.config["SENTRY_SDK"] is False:
        assert "sentry" in app.extensions
    else:
        from sentry_sdk.hub import Hub
        from sentry_sdk.integrations.celery import CeleryIntegration
        from sentry_sdk.integrations.flask import FlaskIntegration

        assert Hub.current and Hub.client
        if app.config["LOGGING_SENTRY_CELERY"]:
            assert Hub.current.get_integration(CeleryIntegration)
        else:
            assert Hub.current.get_integration(FlaskIntegration)


def test_custom_class(pywarnlogger):
    """Test stream handler."""
    from invenio_logging.sentry import InvenioLoggingSentry

    app = Flask("testapp")
    app.config["SENTRY_DSN"] = "http://user:pw@localhost/0"
    app.config["LOGGING_SENTRY_CLASS"] = "invenio_logging.sentry6:Sentry6"
    app.config["SENTRY_SDK"] = False
    InvenioLoggingSentry(app)
    from invenio_logging.sentry6 import Sentry6

    assert isinstance(app.extensions["sentry"], Sentry6)


def test_stream_handler_in_debug(pywarnlogger):
    """Test stream handler."""
    from invenio_logging.sentry import InvenioLoggingSentry

    app = Flask("testapp")
    app.debug = True
    app.config["SENTRY_DSN"] = "http://user:pw@localhost/0"
    app.config["SENTRY_SDK"] = False
    InvenioLoggingSentry(app)
    logger = logging.getLogger("werkzeug")
    assert logging.StreamHandler in [x.__class__ for x in logger.handlers]


def test_sentry_handler_attached_to_app_logger():
    from raven.handlers.logging import SentryHandler

    from invenio_logging.sentry import InvenioLoggingSentry

    app = Flask("testapp")
    app.config["SENTRY_DSN"] = "http://user:pw@localhost/0"
    app.config["SENTRY_SDK"] = False
    InvenioLoggingSentry(app)
    assert any([isinstance(logger, SentryHandler) for logger in app.logger.handlers])


def test_pywarnings(pywarnlogger):
    """Test pywarnings."""
    from invenio_logging.sentry import InvenioLoggingSentry

    app = Flask("testapp")
    app.config.update(
        dict(
            SENTRY_DSN="http://user:pw@localhost/0",
            SENTRY_SDK=False,
            LOGGING_SENTRY_PYWARNINGS=True,
        )
    )
    assert len(pywarnlogger.handlers) == 0
    InvenioLoggingSentry(app)
    assert len(pywarnlogger.handlers) == 1


def test_pywarnings_are_logged_once(pywarnlogger, sentry_emit):
    """Test pywarnings with logging capturing warnings."""
    from raven.handlers.logging import SentryHandler

    from invenio_logging.sentry import InvenioLoggingSentry

    logging.captureWarnings(True)
    app = Flask("testapp")
    app.config.update(
        dict(
            SENTRY_DSN="http://user:pw@localhost/0",
            LOGGING_SENTRY_PYWARNINGS=True,
            SENTRY_SDK=False,
        )
    )
    assert sentry_emit.call_count == 0
    InvenioLoggingSentry(app)
    warnings.warn("sentry test warning")
    assert sentry_emit.call_count == 1


def test_pywarnings_disabled_are_not_logged(pywarnlogger, sentry_emit):
    """Test pywarnings disabled with logging capturing warnings."""
    from raven.handlers.logging import SentryHandler

    from invenio_logging.sentry import InvenioLoggingSentry

    logging.captureWarnings(True)
    app = Flask("testapp")
    app.config.update(
        dict(
            SENTRY_DSN="http://user:pw@localhost/0",
            LOGGING_SENTRY_PYWARNINGS=False,
            SENTRY_SDK=False,
        )
    )
    assert sentry_emit.call_count == 0
    InvenioLoggingSentry(app)
    warnings.warn("sentry test warning")
    assert sentry_emit.call_count == 0


def test_import():
    """Test that raven is not required."""
    with patch("raven.contrib.flask.Sentry") as mod:
        mod.side_effect = ImportError
        # We can import and install extension with out hitting the import
        # unless SENTRY_DSN is set.
        from invenio_logging.sentry import InvenioLoggingSentry

        app = Flask("testapp")
        InvenioLoggingSentry(app)

        app = Flask("testapp")
        app.config["SENTRY_DSN"] = "...."
        app.config["SENTRY_SDK"] = False
        pytest.raises(ImportError, InvenioLoggingSentry, app)


@httpretty.activate
def test_celery():
    """Test celery."""
    from invenio_logging.sentry import InvenioLoggingSentry

    app = Flask("testapp")
    app.config.update(
        dict(
            SENTRY_DSN="http://user:pw@localhost/0",
            SENTRY_TRANSPORT="raven.transport.http.HTTPTransport",
            SENTRY_SDK=False,
            LOGGING_SENTRY_CELERY=True,
            CELERY_ALWAYS_EAGER=True,
            CELERY_RESULT_BACKEND="cache",
            CELERY_CACHE_BACKEND="memory",
            CELERY_EAGER_PROPAGATES_EXCEPTIONS=True,
        )
    )
    InvenioLoggingSentry(app)
    celery = create_celery_app(app)

    @celery.task
    def test_task():
        logger = get_task_logger(__name__)
        logger.error("CELERYTEST")

    httpretty.register_uri(
        httpretty.POST,
        "http://localhost/api/0/store/",
        body='[{"title": "Test"}]',
        content_type="application/json",
        status=200,
    )
    test_task.delay()
    # Give sentry time to send request
    from time import sleep

    sleep(1)
    assert httpretty.last_request().path == "/api/0/store/"


def test_sentry6():
    """Test Sentry 6."""
    from invenio_logging.sentry import InvenioLoggingSentry

    app = Flask("testapp")
    app.config.update(
        dict(
            SENTRY_DSN="http://user:pw@localhost/0",
            SENTRY_SDK=False,
            LOGGING_SENTRY_CLASS="invenio_logging.sentry6:Sentry6",
            SENTRY_USER_ATTRS=["name"],
            SECRET_KEY="CHANGEME",
        )
    )
    InvenioLoggingSentry(app)
    login_manager = LoginManager(app)
    login_manager.user_loader(lambda a: {})

    class User(UserMixin):
        def __init__(self, user_id, name):
            self.id = user_id
            self.name = name

    with app.test_request_context("/"):
        assert app.extensions["sentry"].get_user_info(request) == {}

    with app.test_request_context("/"):
        login_user(User(1, "viggo"))
        assert app.extensions["sentry"].get_user_info(request) == {
            "id": "1",
            "name": "viggo",
        }


def test_request_id_processors():
    """Test the processor which adds request id to the tags context."""
    from invenio_logging.sentry import RequestIdProcessor

    proc = RequestIdProcessor(Mock())
    # Test without request context
    assert proc.process({}) == {}
    assert proc.process({"tags": {}}) == ({"tags": {}})
    assert proc.process({"tags": {"myvar": "val"}}) == {"tags": {"myvar": "val"}}

    # Test with request context
    app = Flask("testpp")
    with app.test_request_context("/"):
        assert proc.process({"tags": {}}) == ({"tags": {}})
        g.request_id = "12"
        assert proc.process({"tags": {}}) == ({"tags": {"request_id": "12"}})



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Logging utils."""

import logging

from flask import g


class AddRequestIdFilter(logging.Filter):
    """Filter for loggers."""

    def filter(self, record):
        """If request_id is set in flask.g, add it to log record."""
        if g and hasattr(g, "request_id"):
            record.request_id = g.request_id
        return True


add_request_id_filter = AddRequestIdFilter()



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""File system logging test."""

from __future__ import absolute_import, print_function

from logging.handlers import RotatingFileHandler
from os.path import exists, join

import pytest
from flask import Flask

from invenio_logging.fs import InvenioLoggingFS


def test_init(tmppath):
    """Test extension initialization."""
    app = Flask("testapp")
    InvenioLoggingFS(app)
    assert "invenio-logging-fs" not in app.extensions

    ext = InvenioLoggingFS()
    app = Flask("testapp")
    ext.init_app(app)
    assert "invenio-logging-fs" not in app.extensions

    logfile = join(tmppath, "test.log")
    app = Flask("testapp")
    app.config.update(dict(LOGGING_FS_LOGFILE=logfile))
    InvenioLoggingFS(app)
    assert "invenio-logging-fs" in app.extensions
    assert app.config["LOGGING_FS_LEVEL"] == "WARNING"


def test_filepath_formatting(tmppath):
    """Test extension initialization."""
    app = Flask("testapp", instance_path=tmppath)
    app.config.update(
        dict(DEBUG=True, LOGGING_FS_LOGFILE="{instance_path}/testapp.log")
    )
    InvenioLoggingFS(app)
    assert app.config["LOGGING_FS_LOGFILE"] == join(tmppath, "testapp.log")
    assert app.config["LOGGING_FS_LEVEL"] == "DEBUG"


def test_missing_dir(tmppath):
    """Test missing dir."""
    app = Flask("testapp")
    filepath = join(tmppath, "invaliddir/test.log")
    app.config.update(dict(LOGGING_FS_LOGFILE=filepath))
    assert pytest.raises(ValueError, InvenioLoggingFS, app)


def test_warnings(tmppath, pywarnlogger):
    """Test extension initialization."""
    app = Flask("testapp", instance_path=tmppath)
    app.config.update(
        dict(
            LOGGING_FS_LOGFILE="{instance_path}/testapp.log",
            LOGGING_FS_PYWARNINGS=True,
            LOGGING_FS_LEVEL="WARNING",
        )
    )
    InvenioLoggingFS(app)
    assert RotatingFileHandler in [x.__class__ for x in pywarnlogger.handlers]


def test_logging(tmppath):
    """Test extension initialization."""
    filepath = join(tmppath, "test.log")
    app = Flask("testapp")
    app.config.update(dict(LOGGING_FS_LOGFILE=filepath, LOGGING_FS_LEVEL="WARNING"))
    InvenioLoggingFS(app)
    # Test delay opening of file
    assert not exists(filepath)
    app.logger.warn("My warning")
    assert exists(filepath)
    app.logger.info("My info")
    # Test log level
    with open(filepath) as fp:
        content = fp.read()
    assert "My warning" in content
    assert "My info" not in content



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Compatibility layer for dealing with Sentry 6.x."""

from __future__ import absolute_import, print_function

from flask import current_app
from flask_login import current_user
from raven.contrib.flask import Sentry


class Sentry6(Sentry):
    """Compatibility layer for Sentry."""

    def get_user_info(self, request):
        """Implement custom getter."""
        if not current_user.is_authenticated:
            return {}

        user_info = {
            "id": current_user.get_id(),
        }

        if "SENTRY_USER_ATTRS" in current_app.config:
            for attr in current_app.config["SENTRY_USER_ATTRS"]:
                if hasattr(current_user, attr):
                    user_info[attr] = getattr(current_user, attr)

        return user_info



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2017-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.


"""Configuration for Invenio-Logging.

Sentry can, in addition to the configuration variables listed, be further
configured with the folllowing configuration variables (see
`Raven <https://docs.sentry.io/clients/python/integrations/flask/#settings>`_
for further details):

- ``SENTRY_AUTO_LOG_STACKS``
- ``SENTRY_EXCLUDE_PATHS``
- ``SENTRY_INCLUDE_PATHS``
- ``SENTRY_MAX_LENGTH_LIST``
- ``SENTRY_MAX_LENGTH_STRING``
- ``SENTRY_NAME``
- ``SENTRY_PROCESSORS``
- ``SENTRY_RELEASE``
- ``SENTRY_SITE_NAME``
- ``SENTRY_TAGS``
- ``SENTRY_TRANSPORT``


.. note::

   Celery does not deal well with the threaded Sentry transport, so you should
   make sure that your **Celery workers** are configured with:

   .. code-block:: python

      SENTRY_TRANSPORT = 'raven.transport.http.HTTPTransport'
"""

# -------
# CONSOLE
# -------
LOGGING_CONSOLE = True
"""Enable logging to the console."""

LOGGING_CONSOLE_PYWARNINGS = None
"""Enable logging of Python warnings to the console.

By default, warnings are logged to the console if the application is in debug
mode, otherwise warnings are not logged.
"""

LOGGING_CONSOLE_LEVEL = None
"""Console logging level.

Set to a valid Python logging level: ``CRITICAL``, ``ERROR``, ``WARNING``,
``INFO``, ``DEBUG``, or ``NOTSET``.
"""


# ----------
# FILESYSTEM
# ----------
LOGGING_FS_LOGFILE = None
"""Enable logging to the filesystem."""

LOGGING_FS_PYWARNINGS = False
"""Enable logging of Python warnings to filesystem logging."""

LOGGING_FS_BACKUPCOUNT = 5
"""Number of rotated log files to keep."""

LOGGING_FS_MAXBYTES = 100 * 1024 * 1024
"""Maximum size of logging file. Default: 100MB."""

LOGGING_FS_LEVEL = None
"""Filesystem logging level.

Set to a valid Python logging level: ``CRITICAL``, ``ERROR``, ``WARNING``,
``INFO``, ``DEBUG``, or ``NOTSET``.
"""


# ------
# SENTRY
# ------

SENTRY_SDK = True
"""Use of sentry-python SDK, if false raven will be used. """

LOGGING_SENTRY_LEVEL = "WARNING"
"""Sentry logging level.

Defaults to only reporting errors and warnings.
"""

LOGGING_SENTRY_PYWARNINGS = False
"""Enable logging of Python warnings to Sentry."""

LOGGING_SENTRY_CELERY = False
"""Configure Celery to send logging to Sentry."""

LOGGING_SENTRY_SQLALCHEMY = False
"""Configure SQL Alchemy to send logging to Sentry."""

LOGGING_SENTRY_REDIS = False
"""Configure REDIS to send logging to Sentry."""

LOGGING_SENTRY_CLASS = None
"""Import path of sentry Flask extension class.

This allows you to customize the Sentry extension class. In particular if you
are logging to Sentry v6, you can set this to
:class:`invenio_logging.sentry6.Sentry6`."""

LOGGING_SENTRY_INIT_KWARGS = None
"""Pass extra options when initializing Sentry instance."""

SENTRY_DSN = None
"""Set SENTRY_DSN environment variable."""

SENTRY_PROCESSORS = (
    "raven.processors.SanitizePasswordsProcessor",
    "invenio_logging.sentry.RequestIdProcessor",
)
"""Default Sentry event processors."""

SENTRY_TRANSPORT = "raven.transport.http.HTTPTransport"
"""Default Sentry transport.

Explicitly set due to Celery incompatibility with threaded transport
(see note above).
"""



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Sentry logging module."""

from __future__ import absolute_import, print_function

import logging
import warnings

import pkg_resources
import six
from flask import g
from werkzeug.utils import import_string

from . import config
from .ext import InvenioLoggingBase

try:
    pkg_resources.get_distribution("raven")
    from raven.processors import Processor
except pkg_resources.DistributionNotFound:

    class Processor(object):
        """Dummy class in case Sentry is not installed.."""

        def __init__(self, *args, **kwargs):
            """Do nothing."""
            pass


class InvenioLoggingSentry(InvenioLoggingBase):
    """Invenio-Logging extension for Sentry."""

    def init_app(self, app):
        """Flask application initialization."""
        self.init_config(app)

        # Only configure Sentry if SENTRY_DSN is set.
        if app.config["SENTRY_DSN"] is None:
            return

        self.install_handler(app)

        app.extensions["invenio-logging-sentry"] = self

    def init_config(self, app):
        """Initialize configuration."""
        for k in dir(config):
            if k.startswith("LOGGING_SENTRY") or k.startswith("SENTRY_"):
                app.config.setdefault(k, getattr(config, k))

    def install_handler(self, app):
        """Install log handler."""
        level = getattr(logging, app.config["LOGGING_SENTRY_LEVEL"])
        logging_exclusions = None
        if not app.config["LOGGING_SENTRY_PYWARNINGS"]:
            logging_exclusions = (
                "raven",
                "gunicorn",
                "south",
                "sentry.errors",
                "django.request",
                "dill",
                "py.warnings",
            )
        if app.config["SENTRY_SDK"]:
            self.install_sentry_sdk_handler(app, logging_exclusions, level)
        else:
            self.install_raven_handler(app, logging_exclusions, level)

        # Werkzeug only adds a stream handler if there's no other handlers
        # defined, so when Sentry adds a log handler no output is
        # received from Werkzeug unless we install a console handler
        # here on the werkzeug logger.
        if app.debug:
            logger = logging.getLogger("werkzeug")
            logger.setLevel(logging.INFO)
            logger.addHandler(logging.StreamHandler())

    def install_sentry_sdk_handler(self, app, logging_exclusions, level):
        """Install sentry-python sdk log handler."""
        import sentry_sdk
        from sentry_sdk import configure_scope
        from sentry_sdk.integrations.celery import CeleryIntegration
        from sentry_sdk.integrations.flask import FlaskIntegration
        from sentry_sdk.integrations.redis import RedisIntegration
        from sentry_sdk.integrations.sqlalchemy import SqlalchemyIntegration

        integrations = [FlaskIntegration()]
        init_kwargs = {}
        if app.config["LOGGING_SENTRY_CELERY"]:
            integrations.append(CeleryIntegration())
        if app.config["LOGGING_SENTRY_SQLALCHEMY"]:
            integrations.append(SqlalchemyIntegration())
        if app.config["LOGGING_SENTRY_REDIS"]:
            integrations.append(RedisIntegration())
        if app.config["LOGGING_SENTRY_INIT_KWARGS"]:
            init_kwargs = app.config["LOGGING_SENTRY_INIT_KWARGS"]

        sentry_sdk.init(
            dsn=app.config["SENTRY_DSN"],
            in_app_exclude=logging_exclusions,
            integrations=integrations,
            before_send=self.add_request_id_sentry_python,
            **init_kwargs
        )
        with configure_scope() as scope:
            scope.level = level

    def install_raven_handler(self, app, logging_exclusions, level):
        """Install raven log handler."""
        warnings.warn(
            "The Raven library will be depricated.", PendingDeprecationWarning
        )
        from raven.contrib.celery import register_logger_signal, register_signal
        from raven.contrib.flask import Sentry
        from raven.handlers.logging import SentryHandler

        cls = app.config["LOGGING_SENTRY_CLASS"]
        if cls:
            if isinstance(cls, six.string_types):
                cls = import_string(cls)
        else:
            cls = Sentry
        sentry = cls(
            app,
            logging=True,
            level=level,
            logging_exclusions=logging_exclusions,
        )
        app.logger.addHandler(SentryHandler(client=sentry.client, level=level))

        # Capture warnings from warnings module
        if app.config["LOGGING_SENTRY_PYWARNINGS"]:
            self.capture_pywarnings(SentryHandler(sentry.client))

        # Setup Celery logging to Sentry
        if app.config["LOGGING_SENTRY_CELERY"]:
            try:
                register_logger_signal(sentry.client, loglevel=level)
            except TypeError:
                # Compatibility mode for Raven<=5.1.0
                register_logger_signal(sentry.client)
            register_signal(sentry.client)

    def add_request_id_sentry_python(self, event, hint):
        """Add the request id as a tag."""
        if g and hasattr(g, "request_id"):
            tags = event.get("tags") or []
            tags.append(["request_id", g.request_id])
            event["tags"] = tags
        return event


class RequestIdProcessor(Processor):
    """Sentry event request processor for adding the request id as a tag."""

    def process(self, data, **kwargs):
        """Process event data."""
        data = super(RequestIdProcessor, self).process(data, **kwargs)
        if g and hasattr(g, "request_id"):
            tags = data.get("tags", {})
            tags["request_id"] = g.request_id
            data["tags"] = tags
        return data



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test example app."""

import os
import signal
import subprocess
import time
from os.path import abspath, dirname, join

import pytest


@pytest.yield_fixture
def example_app():
    """Example app fixture."""
    current_dir = os.getcwd()

    # Go to example directory
    project_dir = dirname(dirname(abspath(__file__)))
    exampleappdir = join(project_dir, "examples")
    os.chdir(exampleappdir)

    # Setup application
    assert subprocess.call("./app-setup.sh", shell=True) == 0

    # Setup fixtures
    assert subprocess.call("./app-fixtures.sh", shell=True) == 0

    # Start example web app
    cmd = "FLASK_APP=app.py flask run --debugger -p 5001"
    webapp = subprocess.Popen(
        cmd, stdout=subprocess.PIPE, preexec_fn=os.setsid, shell=True
    )
    time.sleep(5)
    # return webapp
    yield webapp

    # Stop server
    os.killpg(webapp.pid, signal.SIGTERM)

    # Tear down example app
    subprocess.call("./app-teardown.sh", shell=True)

    # Return to the original directory
    os.chdir(current_dir)


def test_example_app(example_app):
    """Test example app."""
    # Testing get index page
    cmd = "curl http://127.0.0.1:5001/"
    output = subprocess.check_output(cmd, shell=True).decode("utf-8")
    assert "Welcome to Invenio-Logging!" == output



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Warnings capturing tests."""

from __future__ import absolute_import, print_function

import logging

from flask import Flask

from invenio_logging.ext import InvenioLoggingBase


def test_init(pywarnlogger):
    """Test extension initialization."""
    app = Flask("testapp")
    ext = InvenioLoggingBase(app)
    assert len(pywarnlogger.handlers) == 0

    # Capture warnings.
    handler = logging.StreamHandler()
    ext.capture_pywarnings(handler)
    assert len(pywarnlogger.handlers) == 1

    # Don't install the same handler twice (i.e. prevent multiple Flask apps
    # to install the same handlers and thus receiving double notifications)
    handler = logging.StreamHandler()
    ext.capture_pywarnings(handler)
    assert len(pywarnlogger.handlers) == 1

    # Different types of handlers are welcome
    handler = logging.NullHandler()
    ext.capture_pywarnings(handler)
    assert len(pywarnlogger.handlers) == 2



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio filesystem logging module.

This extension is enabled by default and automatically installed via
``invenio_base.apps`` and ``invenio_base.api_apps`` entry points.
"""

from __future__ import absolute_import, print_function

import logging

from . import config
from .ext import InvenioLoggingBase
from .utils import add_request_id_filter


class InvenioLoggingConsole(InvenioLoggingBase):
    """Invenio-Logging extension for console."""

    def init_app(self, app):
        """Flask application initialization."""
        self.init_config(app)

        if not app.config["LOGGING_CONSOLE"]:
            return

        self.install_handler(app)

        app.extensions["invenio-logging-console"] = self

    def init_config(self, app):
        """Initialize config."""
        app.config.setdefault("LOGGING_CONSOLE", True)
        app.config.setdefault("LOGGING_CONSOLE_PYWARNINGS", app.debug)
        for k in dir(config):
            if k.startswith("LOGGING_CONSOLE"):
                app.config.setdefault(k, getattr(config, k))

    def install_handler(self, app):
        """Install logging handler."""
        # Configure python logging
        if app.config["LOGGING_CONSOLE_PYWARNINGS"]:
            self.capture_pywarnings(logging.StreamHandler())

        if app.config["LOGGING_CONSOLE_LEVEL"] is not None:
            for h in app.logger.handlers:
                h.setLevel(app.config["LOGGING_CONSOLE_LEVEL"])

        # Add request_id to log record
        app.logger.addFilter(add_request_id_filter)
